/*
    Irchat - Infrared chat on TI89 calculators
    Copyright (C) 2003  Sbastien Bourdeauducq

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "config.h"

#include <compat.h>
#include <dialogs.h>
#include <kbd.h>
#include <statline.h>
#include <string.h>
#include <system.h>
#include <wingraph.h>

#include "comm_lowlevel.h"
#include "comm.h"
#include "icon2.h"
#include "locale.h"
#include "log.h"

unsigned short chatroom_exit;

WIN_RECT MessageArea, EditorArea;

char EditorText[MESSAGE_LENGTH+1];

void clear_screen(void) {
  WinClr(DeskTop);
  WinLineNC(DeskTop, &(WIN_RECT){0, MessageArea.y1+1, MessageArea.x1, MessageArea.y1+1});
  EditorText[0] = 0;
  WinFont(DeskTop, F_6x8);
}

void hardware_probe(void) {
  while(!hwtest()) {
    if (DlgMessage(HW_PROBLEM, HW_NODETECT_RETRY, BT_YES, BT_NO) == KEY_ESC) {
      chatroom_exit = TRUE;
      return;
    }
    ST_busy(ST_IDLE);
  }
}

void print_message(const char *message) {
  WinScrollV(DeskTop, &MessageArea, 9);
  WinStrXY(DeskTop, 0, MessageArea.y1-9, message);
}

void received_message(const char *message) {
  char buffer[MESSAGE_LENGTH+3];
  log_received_message(message);
  buffer[0] = 25;
  buffer[1] = ' ';
  memcpy(&buffer[2], message, MESSAGE_LENGTH+1);
  print_message(buffer);
  ST_eraseHelp();
}

void editor_insert(char *s) {
  if(strlen(EditorText)+strlen(s) > MESSAGE_LENGTH) return;
  strcat(EditorText, s);
  WinStrXY(DeskTop, (strlen(EditorText)-1)*6, EditorArea.y0+1, s);
}

void editor_clear(void) {
  WinFill(DeskTop, &EditorArea, A_REVERSE);
  memset(EditorText, 0, MESSAGE_LENGTH+1);
}

void editor_backspace(void) {
  unsigned long i;
  i = strlen(EditorText);
  if(i==0) return;
  i--;
  EditorText[i] = 0;
  WinFill(DeskTop, &(WIN_RECT){EditorArea.x0+(i*6), EditorArea.y0, EditorArea.x0+((i+1)*6), EditorArea.y0+8}, A_REVERSE);
}

void editor_send_message(void) {
  char buffer[MESSAGE_LENGTH+3];
  ST_showHelp(SENDING_MESSAGE);
  if(send_message(EditorText)) {
    log_sent_message(EditorText);
    memset(buffer, 0, MESSAGE_LENGTH+3);
    buffer[0] = 26;
    buffer[1] = ' ';
    memcpy(&buffer[2], EditorText, MESSAGE_LENGTH+1);
    print_message(buffer);
    editor_clear();
  } else {
    DlgMessage(COMM_ERROR, MESSAGE_NOT_RECEIVED, BT_OK, BT_NONE);
    ST_busy(ST_IDLE);
  }
  ST_eraseHelp();
}

void chatroom_process_key(unsigned short key) {
  char buf[2];
  ST_eraseHelp();
  switch(key) {
    case (KEY_DIAMOND+KEY_CLEAR):
      clear_screen();
      break;
    case KEY_CLEAR:
      editor_clear();
      break;
    case KEY_BACKSPACE:
      editor_backspace();
      break;
    case KEY_ENTER:
      editor_send_message();
      break;
    case KEY_ESC:
      chatroom_exit = TRUE;
      break;
    default:
      if(((key >= 'a') && (key <= 'z')) || ((key >= 'A') && (key<= 'Z'))
        || ((key >= '0') && (key <= '9')) || (key == '=') || (key == '(')
        || (key == ')') || (key == ')') || (key == ',') || (key == '^')
        || (key == '|') || (key == '.') || (key == '/') || (key == '*')
        || (key == '+') || (key == '-') || (key == ' ') || (key == ':')
        || (key == '"') || (key == '{') || (key == '}') || (key == '[')
        || (key == ']') || (key == ';') || (key == '<') || (key == '>')
        || (key == '\'') || (key == '\\')) {
          buf[0] = key;
          buf[1] = 0;
          editor_insert(buf);
      }
      break;
  }
}

void chatroom_start(void) {
  void *kbq;
  unsigned short key;
  kbq = kbd_queue();

  chatroom_exit = FALSE;

  MessageArea.x0 = 0;
  MessageArea.y0 = 0;
  MessageArea.x1 = WinWidth(DeskTop);
  MessageArea.y1 = WinHeight(DeskTop)-11;

  EditorArea.x0 = 0;
  EditorArea.y0 = MessageArea.y1+2;
  EditorArea.x1 = MessageArea.x1;
  EditorArea.y1 = WinHeight(DeskTop);

  clear_screen();
  WinBitmapPut(DeskTop, 51, 35, &icon2, A_REPLACE);
  WinFont(DeskTop, F_4x6);
  WinStrXY(DeskTop, 0, 55, CHATROOM_MSG1);
  WinStrXY(DeskTop, 0, 62, CHATROOM_MSG2);
  WinStrXY(DeskTop, 0, 69, CHATROOM_MSG3);
  WinStrXY(DeskTop, 0, 76, CHATROOM_MSG4);
  WinFont(DeskTop, F_6x8);
  memset(EditorText, 0, MESSAGE_LENGTH+1);
  ST_eraseHelp();
  open_log();

  while(!chatroom_exit) {
    check_incoming();
    if(!OSdequeue(&key, kbq)) chatroom_process_key(key);
    idle();
  }

  close_log();
}
