/*
    Irchat - Infrared chat on TI89 calculators
    Copyright (C) 2003  Sbastien Bourdeauducq

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "config.h"

#include <mem.h>
#include <statline.h>
#include <system.h>

#include "comm_lowlevel.h"
#include "comm.h"
#include "chatroom.h"
#include "locale.h"

#define ACKSTR_LENGTH 4

char ackstr[ACKSTR_LENGTH] = "OK_1";

#define MSGBUFFER_LENGTH MESSAGE_LENGTH+1 /* MESSAGE_LENGTH bytes for the message itself + 1 byte of checksum */
#define ACKBUFFER_LENGTH ACKSTR_LENGTH+1  /* ACKSTR_LENGTH bytes for the acknowledgement string + 1 byte of checksum confirmation */

char msgbuffer[MSGBUFFER_LENGTH];
char ackbuffer[ACKBUFFER_LENGTH];

char buffer[MESSAGE_LENGTH+1];

void check_incoming(void) {
  unsigned char computed_checksum;
  unsigned char i, b;
  if(ir_read()) {
    /* We have the leader pulse, get the packet */
    ST_showHelp(RECEIVING_MESSAGE);
    if(!uart_recv(msgbuffer, MSGBUFFER_LENGTH)) {
      hardware_probe();
      return;
    }
    b = TRUE;
    for(i=0;i<MSGBUFFER_LENGTH;i++) if(msgbuffer[i] != 0) {
      b = FALSE;
      break;
    }
    if(b) {
      ST_eraseHelp();
      return;
    }
    /* Verify the checksum */
    computed_checksum = 0;
    for(i=1; i<MESSAGE_LENGTH+1; i++) computed_checksum ^= msgbuffer[i];
    if(computed_checksum != msgbuffer[0]) {
      /* Invalid checksum */
      ST_eraseHelp();
      return;
    }
    /* Send the acknowledgement */
    memcpy(ackbuffer, ackstr, ACKSTR_LENGTH);
    ackbuffer[ACKSTR_LENGTH] = computed_checksum;
    if (!uart_send(ackbuffer, ACKBUFFER_LENGTH)) {
      hardware_probe();
      return;
    }
    memcpy(buffer, &msgbuffer[1], MESSAGE_LENGTH);
    buffer[MESSAGE_LENGTH] = 0;
    received_message(buffer);
  }
}

unsigned char send_message(const char *message) {
  unsigned char i;
  msgbuffer[0] = 0;
  memcpy(&msgbuffer[1], message, MESSAGE_LENGTH);
  for(i=1; i<MESSAGE_LENGTH+1; i++) msgbuffer[0] ^= msgbuffer[i];
  if(!uart_send(msgbuffer, MSGBUFFER_LENGTH)) {
    hardware_probe();
    return FALSE;
  }
  OSFreeTimer(USER_TIMER);
  OSRegisterTimer(USER_TIMER, 20);
  while(!ir_read()) if (OSTimerExpired(USER_TIMER)) return FALSE;
  uart_recv(ackbuffer, ACKBUFFER_LENGTH);
  if(memcmp(ackbuffer, ackstr, ACKSTR_LENGTH) != 0) return FALSE;
  if(ackbuffer[ACKSTR_LENGTH] != msgbuffer[0]) return FALSE;
  return TRUE;
}
