/*
    Irchat - Infrared chat on TI89 calculators
    Copyright (C) 2003  Sbastien Bourdeauducq

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "config.h"

#include <dialogs.h>
#include <mem.h>
#include <stdio.h>
#include <system.h>

#include "locale.h"

/* Thanks to Olivier Armand (ExtendeD) for these functions and to all the people */
/* who helped me with them on TICT programming board */

#define IsClockOn _rom_call(unsigned char,(void),5F7)
#define DateAndTime_Get _rom_call(void,(unsigned short*,unsigned short*,unsigned short*,unsigned short*,unsigned short*,unsigned short*),5F3)
#define DateStr _rom_call(void,(unsigned short,unsigned short,unsigned short,unsigned char,char*),5FE)
#define TimeStr _rom_call(void,(unsigned short,unsigned short,unsigned short,unsigned char,unsigned char,char*),601)
#define DateFormat_Get _rom_call(unsigned char,(void),5FC)
#define TimeFormat_Get _rom_call(unsigned char,(void),5FF)

unsigned short IsClockAvailable()
{
  if(((BASECODE_PARM_BLOCK *)EX_getBasecodeParmBlock())->releaseVersionMinor < 8) return FALSE;
  return IsClockOn();
}

char logfilename[9];

FILE *f = NULL;

void open_log()
{
  unsigned short year, month, day, hour, minute, second;
  char DateBuf[9];
  char TimeBuf[12];
  if(logfilename[0] != 0) {
    f = fopen(logfilename, "at");
    if(!f) DlgMessage(LOGGING, UNABLE_TO_OPEN_LOGFILE, BT_OK, BT_NONE);
  }
  else {
    f = NULL;
  }
  if(f) {
    if(IsClockAvailable()) {
      DateAndTime_Get(&year, &month, &day, &hour, &minute, &second);
      DateBuf[0] = 0;
      TimeBuf[0] = 0;
      DateStr(year, month, day, DateFormat_Get(), DateBuf);
      TimeStr(hour, minute, second, FALSE, TimeFormat_Get(), TimeBuf);
      fprintf(f, BEGIN_LOGGING_TIMESTAMPED, DateBuf, TimeBuf);
    }
    else {
      fputs(BEGIN_LOGGING, f);
    }
  }
}

void log_received_message(const char *message)
{
  if(!f) return;
  fprintf(f, "\x19 %s\n", message);
}

void log_sent_message(const char *message)
{
  if(!f) return;
  fprintf(f, "\x1A %s\n", message);
}

void close_log()
{
  unsigned short year, month, day, hour, minute, second;
  char DateBuf[9];
  char TimeBuf[12];
  if(!f) return;
  if(IsClockAvailable()) {
    DateAndTime_Get(&year, &month, &day, &hour, &minute, &second);
    DateBuf[0] = 0;
    TimeBuf[0] = 0;
    DateStr(year, month, day, DateFormat_Get(), DateBuf);
    TimeStr(hour, minute, second, FALSE, TimeFormat_Get(), TimeBuf);
    fprintf(f, END_LOGGING_TIMESTAMPED, DateBuf, TimeBuf);
  }
  else {
    fputs(END_LOGGING, f);
  }
  fclose(f);
  f = NULL;
}
